@@ -82,7 +82,7 @@ int main(int argc, char *argv[])
 {
 	char *file = NULL;
 	char *lang;
-	int runcd = 0;
+	int runcd = 2;
 	char* loadst = NULL;
 	int i;
 	int  skipCountTable[9] 	= { 0,1,3,2,4,7,10,15,17 };
@@ -166,6 +166,10 @@ int main(int argc, char *argv[])
 				linesInterlace_user = iLace;
 			}
 		}
+		else if (!strcmp(argv[i], "-abbeyfix"))
+		{
+			enableAbbeyHack=true;
+		}
 		else if (!strcmp(argv[i], "-h") ||
 			 !strcmp(argv[i], "-help") ||
 			 !strcmp(argv[i], "--help")) {
@@ -184,6 +188,7 @@ int main(int argc, char *argv[])
 							"\t-frameskip 0-8\tFrame skipping ratio\n"
 							"\t-cyclemult 0-10\tCPU cycle multiplier\n"
 							"\t-ilace 0,1,3,7\tInterlace lines\n"
+							"\t-abbeyfix\tEnabling Abbeyfix\n"
 							"\tFILE\t\tCdRom file\n"));
 			 return 0;
 		} else file = argv[i];
@@ -193,7 +198,7 @@ int main(int argc, char *argv[])
 	gp2x_init(1000, 16, 11025, 16, 1, 60, 1);
 
 #ifdef MAEMO_CHANGES
-	hildon_init (&argc, &argv);
+	hildon_init (&argc, &argv,file,loadst);
 #endif
 
 	if (SysInit() == -1) return 1;
