@@ -19,6 +19,18 @@ unsigned short* screenbuffer = NULL;
 
 unsigned long keys = 0;
 
+int keymap[65536];
+char string [255];
+char *loadst;
+char *file;
+
+void hildon_quit()
+{
+	gp2x_deinit();
+	gtk_main_quit();
+	exit(0);
+}
+
 static void
 window_key_proxy (GtkWidget *widget,
 		     GdkEventKey *event,
@@ -26,56 +38,87 @@ window_key_proxy (GtkWidget *widget,
 {
 	unsigned long key = 0;
 	
-	switch (event->keyval) {
-		case GDK_Left:
-			key = GP2X_LEFT;
-			break;
-		case GDK_Right:
-			key = GP2X_RIGHT;
-			break;
-		case GDK_Up:
-			key = GP2X_UP;
-			break;
-		case GDK_Down:
-			key = GP2X_DOWN;
-			break;
+	//printf("%d\n",event->keyval);		
 
-		case GDK_x:
-			key = GP2X_B;
-			break;
-		case GDK_z:
-			key = GP2X_X;
-			break;
-		case GDK_s:
-			key = GP2X_Y;
-			break;
-		case GDK_a:
-			key = GP2X_A;
-			break;
-			
-		case GDK_space:
-			key = GP2X_SELECT;
-			break;
-		case GDK_KP_Enter:
-		case GDK_Return:
-			key = GP2X_START;
-			break;
-
-		case GDK_q:
-			key = GP2X_VOL_DOWN;
-			break;
-		case GDK_r:
-			key = GP2X_VOL_UP;
-			break;
-		case GDK_w:
-			key = GP2X_L;
-			break;
-		case GDK_e:
-			key = GP2X_R;
-			break;
-
-		default:
-			key = 0;
+	switch(keymap[event->keyval]){
+	case 0:
+			key = GP2X_LEFT;break;
+	case 1:
+			key = GP2X_RIGHT;break;
+	case 2:
+			key = GP2X_UP;break;
+	case 3:
+			key = GP2X_DOWN;break;
+	case 4:
+			key = GP2X_B;break;
+	case 5:
+			key = GP2X_X;break;
+	case 6:
+			key = GP2X_Y;break;
+	case 7:
+			key = GP2X_A;break;
+	case 8:
+			key = GP2X_SELECT;break;
+	case 9:
+			key = GP2X_START;break;
+	case 10:
+			key = GP2X_VOL_DOWN;break;
+	case 11:
+			key = GP2X_VOL_UP;break;
+	case 12:
+			key = GP2X_L;break;
+	case 13:
+			key = GP2X_R;break;
+	case 14:
+		hildon_quit();break;
+	case 15:
+			if (event->type == GDK_KEY_PRESS){
+			keys |= GP2X_LEFT;
+			keys |= GP2X_UP;
+			}else if (event->type == GDK_KEY_RELEASE){
+			keys &= ~GP2X_LEFT;
+			keys &= ~GP2X_UP;
+			}
+			key = 0;break;
+	case 16:
+			if (event->type == GDK_KEY_PRESS){
+			keys |= GP2X_RIGHT;
+			keys |= GP2X_UP;
+			}else if (event->type == GDK_KEY_RELEASE){
+			keys &= ~GP2X_RIGHT;
+			keys &= ~GP2X_UP;
+			}
+			key = 0;break;
+	case 17:
+			if (event->type == GDK_KEY_PRESS){
+			keys |= GP2X_LEFT;
+			keys |= GP2X_DOWN;
+			}else if (event->type == GDK_KEY_RELEASE){
+			keys &= ~GP2X_LEFT;
+			keys &= ~GP2X_DOWN;
+			}
+			key = 0;break;
+	case 18:
+			if (event->type == GDK_KEY_PRESS){
+			keys |= GP2X_RIGHT;
+			keys |= GP2X_DOWN;
+			}else if (event->type == GDK_KEY_RELEASE){
+			keys &= ~GP2X_RIGHT;
+			keys &= ~GP2X_DOWN;
+			}
+			key = 0;break;
+	case 19:
+		snprintf(string,254,"%s.%s.save",file,loadst);
+		SaveState(string);
+		key = 0;break;
+	break;
+	case 20:
+		snprintf(string,254,"%s.%s.save",file,loadst);
+		LoadState(string);
+		key = 0;break;
+	break;
+	default:
+			key = 0;break;
 	}
 	
 	if (event->type == GDK_KEY_PRESS) {
@@ -85,20 +128,33 @@ window_key_proxy (GtkWidget *widget,
 		keys &= ~key;
 	}
 
-	//printf("Key 0x%x %s (0x%x)\n", key, event->type == GDK_KEY_PRESS ? "pressed" : "released", keys);
 }
 
-void hildon_quit()
-{
-	gp2x_deinit();
-	gtk_main_quit();
-	exit(0);
-}
 
-void hildon_init(int *argc, char ***argv)
+void hildon_init(int *argc, char ***argv,char *fl,char* lst)
 {
+	file=fl;
+	loadst=lst;
+
+	// read key config
+	FILE* pFile;
+	pFile = fopen("/home/opt/psx4m/keys", "r"); // assume the file exists and has data
+	int ch;
+	for (int i=0;i<65536;i++)
+		keymap[i]=0;
+	if (NULL != pFile) {
+		for(int i=0;i<21;i++){
+		try{
+			fscanf(pFile, "%i",&ch);
+			keymap[ch]=i;
+			}catch(int e){
+			keymap[i]=0;
+			}
+		}
+		fclose(pFile);
+	}
+
 	gtk_init (argc, argv);
-
 	window = hildon_stackable_window_new ();
 	gtk_widget_realize (window);
 	gtk_window_fullscreen (GTK_WINDOW(window));
@@ -200,4 +256,3 @@ void gp2x_printfchar15(gp2x_font *f, unsigned char c)
           dst+=(image->bpl>>1)-(f->w);
          }
 }
-
